@@ -1,5 +1,5 @@
 /*
- * $Id: header_f.h 250 2005-04-10 20:57:28Z calrissian $
+ * $Id: header_f.h 369 2005-11-20 18:40:24Z calrissian $
  *
  * Copyright (C) 2002-2004 Fhg Fokus
  * Copyright (C) 2004-2005 Nils Ohlmeier
@@ -20,23 +20,44 @@
 #ifndef SIPSAK_HEADER_H
 #define SIPSAK_HEADER_H
 
+#include "shoot.h"
+
 void add_via(char *mes);
 
 void cpy_vias(char *reply, char *dest);
 
 void cpy_to(char *reply, char *dest);
 
-void set_maxforw(char *mes);
+void set_maxforw(char *mes, int value);
 
 void uri_replace(char *mes, char *uri);
 
 void set_cl(char* mes, int contentlen);
 
-void build_ack(char *invite, char *reply, char *ack);
+int get_cl(char* mes);
+
+int find_lr_parameter(char *rr_line);
+
+void cpy_rr(char* src, char *dst, int route);
+
+void build_ack(char *invite, char *reply, char *dest, 
+			struct sipsak_regexp *reg);
 
 void warning_extract(char *message);
 
 int cseq(char *message);
 
 void increase_cseq(char *message);
+
+void parse_uri(char *uri, char **scheme, char **user, char **host, int *port);
+
+char* uri_from_contact(char *message);
+
+void new_branch(char *message);
+
+void new_transaction(char *message);
+
+void print_message_line(char *message);
+
+inline char* get_body(char *mes);
 #endif
