@@ -1,6 +1,6 @@
 /* 
 
-$Id: md5.c 248 2005-04-10 20:25:48Z calrissian $
+$Id: md5.c 361 2005-11-16 23:35:45Z calrissian $
 
 MD5C.C - RSA Data Security, Inc., MD5 message-digest algorithm
 
@@ -30,16 +30,14 @@ documentation and/or software.
 # include "config.h"
 #endif
 
-#ifndef HAVE_OPENSSL_MD5_H
+#include "md5.h"
+
+#ifndef HAVE_EXTERNAL_MD5
 
 #ifdef HAVE_STRING_H
 # include <string.h>
 #endif
 
-#include "md5global.h"
-#include "md5.h"
-
-
 #define USE_MEM
 
 /* Constants for MD5Transform routine.
@@ -363,4 +361,4 @@ unsigned int len;
 #endif
 }
 
-#endif /* ! HAVE_OPENSSL_MD5_H */
+#endif /* ! HAVE_EXTERNAL_MD5 */
