@@ -1,5 +1,5 @@
 /*
- * $Id: shoot.h 250 2005-04-10 20:57:28Z calrissian $
+ * $Id: shoot.h 367 2005-11-20 17:51:43Z calrissian $
  *
  * Copyright (C) 2002-2004 Fhg Fokus
  * Copyright (C) 2004-2005 Nils Ohlmeier
@@ -27,11 +27,37 @@
 #ifdef HAVE_NETINET_IP_H
 # ifdef HAVE_NETINET_IP_ICMP_H
 #  ifdef HAVE_NETINET_UDP_H
-#   define RAW_SUPPORT
+#   ifndef HAVE_CYGWIN_ICMP_H
+#    define RAW_SUPPORT
+#   endif
 #  endif
 # endif
 #endif
 
-void shoot(char *buff);
+#define LPORT_STR_LEN 6
+
+struct sipsak_regexp {
+	regex_t redexp;
+	regex_t proexp;
+	regex_t okexp;
+	regex_t tmhexp;
+	regex_t errexp;
+	regex_t authexp;
+	regex_t replyexp;
+};
+
+enum usteps { 
+	REG_REP,
+	INV_RECV,
+	INV_OK_RECV,
+	INV_ACK_RECV,
+	MES_RECV, 
+	MES_OK_RECV,
+	UNREG_REP
+};
+
+int inv_trans;
+
+void shoot(char *buff, int buff_size);
 
 #endif
