@@ -1,5 +1,5 @@
 /*
- * $Id: sipsak.c 270 2005-06-29 18:52:49Z calrissian $
+ * $Id: sipsak.c 361 2005-11-16 23:35:45Z calrissian $
  *
  * Copyright (C) 2002-2004 Fhg Fokus
  * Copyright (C) 2004-2005 Nils Ohlmeier
@@ -56,6 +56,7 @@
 #endif
 
 #include "helper.h"
+#include "header_f.h"
 #include "shoot.h"
 #include "exit_code.h"
 
@@ -76,11 +77,11 @@ void print_version() {
 	printf(
 		" shoot  : sipsak [-f FILE] [-L] -s SIPURI\n"
 		" trace  : sipsak -T -s SIPURI\n"
-		" usrloc : sipsak -U [-I|M] [-b NUMBER] [-e NUMBER] [-x NUMBER] [-z] -s SIPURI\n"
+		" usrloc : sipsak -U [-I|M] [-b NUMBER] [-e NUMBER] [-x NUMBER] [-z NUMBER] -s SIPURI\n"
 		" usrloc : sipsak -I|M [-b NUMBER] [-e NUMBER] -s SIPURI\n"
 		" usrloc : sipsak -U [-C SIPURI] [-x NUMBER] -s SIPURI\n"
-		" message: sipsak -M [-B STRING] [-O STRING] -s SIPURI\n"
-		" flood  : sipsak -F [-c NUMBER] -s SIPURI\n"
+		" message: sipsak -M [-B STRING] [-O STRING] [-c SIPURI] -s SIPURI\n"
+		" flood  : sipsak -F [-e NUMBER] -s SIPURI\n"
 		" random : sipsak -R [-t NUMBER] -s SIPURI\n\n"
 		" additional parameter in every mode:\n"
 		);
@@ -111,10 +112,8 @@ void print_long_help() {
 		"  --sleep=NUMBER             sleep number ms before sending next request\n"
 		);
 	printf("  --expires=NUMBER           the expires header field value (default: 15)\n"
-		"  --remove-bindings          activates randomly removing of user bindings\n"
+		"  --remove-bindings=NUMBER   activates randomly removing of user bindings\n"
 		"  --flood-mode               activates the flood mode\n"
-		"  --cseq-max=NUMBER          the maximum CSeq number for flood mode "
-			"(default: 2^31)\n"
 		"  --random-mode              activates the random modues (dangerous)\n"
 		"  --trash-chars=NUMBER       the maximum number of trashed character in random mode\n"
 		"                               (default: request length)\n"
@@ -144,6 +143,11 @@ void print_long_help() {
 		"  --search=REGEXP            search for a RegExp in replies and return error\n"
 		"                             on failfure\n"
 		"  --timing                   print the timing informations at the end\n"
+		"  --symmetric                send and received on the same port\n"
+		"  --from=SIPURI              use the given uri as From in MESSAGE\n"
+		"  --timeout-factor=NUMBER    timeout multiplier for INVITE transactions\n"
+		"                             and reliable transports (default: 64)\n"
+		"  --transport=STRING         specify transport to be used\n"
 		);
 	exit_code(0);
 }
@@ -170,11 +174,10 @@ void print_help() {
 		"  -e NUMBER         the ending numer of the appendix to the user name\n"
 		"  -o NUMBER         sleep number ms before sending next request\n"
 		"  -x NUMBER         the expires header field value (default: 15)\n"
-		"  -z                activates randomly removing of user bindings\n"
+		"  -z NUMBER         activates randomly removing of user bindings\n"
 		"  -F                activates the flood mode\n"
 		);
 	printf(
-		"  -c NUMBER         the maximum CSeq number for flood mode (default: 2^31)\n"
 		"  -R                activates the random modues (dangerous)\n"
 		"  -t NUMBER         the maximum number of trashed character in random mode\n"
 		"                      (default: request length)\n"
@@ -205,6 +208,11 @@ void print_help() {
 		"  -O STRING         Content-Disposition value\n"
 		"  -P NUMBER         Number of processes to start\n"
 		"  -A                print timing informations\n"
+		"  -S                use same port for receiving and sending\n"
+		"  -c SIPURI         use the given uri as From in MESSAGE\n"
+		"  -D NUMBER         timeout multiplier for INVITE transactions\n"
+		"                    and reliable transports (default: 64)\n"
+		"  -E STRING         specify transport to be used\n"
 		);
 		exit_code(0);
 }
@@ -213,10 +221,11 @@ int main(int argc, char *argv[])
 {
 	FILE	*pf;
 	char	buff[BUFSIZE];
-	int	length, c, i, j;
-	char	*delim, *delim2;
+	int		c, i, port;
+	unsigned int tsp;
+	char	*scheme, *user, *host, *backup;
 	pid_t 	pid;
-	struct timespec ts;
+	struct 	timespec ts;
 	int 	upp;
 
 #ifdef HAVE_GETOPT_LONG
@@ -235,9 +244,8 @@ int main(int argc, char *argv[])
 		{"appendix-end", 1, 0, 'e'},
 		{"sleep", 1, 0, 'o'},
 		{"expires", 1, 0, 'x'},
-		{"remove-bindings", 0, 0, 'z'},
+		{"remove-bindings", 1, 0, 'z'},
 		{"flood-mode", 0, 0, 'F'},
-		{"cseq-max", 1, 0, 'c'},
 		{"random-mode", 0, 0, 'R'},
 		{"trash-chars", 1, 0, 't'},
 		{"local-port", 1, 0, 'l'},
@@ -262,100 +270,137 @@ int main(int argc, char *argv[])
 		{"auth-username", 1, 0, 'u'},
 		{"no-crlf", 0, 0, 'L'},
 		{"timing", 0, 0, 'A'},
+		{"symmetric", 0, 0, 'S'},
+		{"from", 1, 0, 'c'},
+		{"timeout-factor", 1, 0, 'D'},
+		{"transport", 1, 0, 'E'},
 		{0, 0, 0, 0}
 	};
 #endif
 	/* some initialisation to be shure */
 	file_b=uri_b=trace=lport=usrloc=flood=verbose=randtrash=trashchar = 0;
 	warning_ext=rand_rem=nonce_count=replace_b=invite=message = 0;
-	sleep_ms=empty_contact=nagios_warn=timing = 0;
+	sleep_ms=empty_contact=nagios_warn=timing=outbound_proxy=symmetric = 0;
 	namebeg=nameend=maxforw= -1;
 	numeric=via_ins=redirects=fix_crlf=processes = 1;
 	username=password=replace_str=hostname=contact_uri=mes_body = NULL;
-	con_dis=auth_username = NULL;
+	con_dis=auth_username=from_uri = NULL;
+	scheme = user = host = backup = req = rep = rec = NULL;
 	re = NULL;
-	address = 0;
-	rport = 0;
+	address= 0;
+	transport=tsp = 0;
+	rport = port = 0;
 	expires_t = USRLOC_EXP_DEF;
+	inv_final = 64 * SIP_T1;
 	memset(buff, 0, BUFSIZE);
-	memset(confirm, 0, BUFSIZE);
-	memset(ack, 0, BUFSIZE);
 	memset(fqdn, 0, FQDN_SIZE);
-	memset(messusern, 0, FQDN_SIZE);
 
-	if (argc==1) print_help();
+	if (argc==1) {
+		print_help();
+	}
 
 	/* lots of command line switches to handle*/
 #ifdef HAVE_GETOPT_LONG
-	while ((c=getopt_long(argc, argv, "a:Ab:B:c:C:de:f:Fg:GhH:iIl:Lm:MnNo:O:p:P:q:r:Rs:t:Tu:UvVwW:x:z", l_opts, &option_index)) != EOF){
+	while ((c=getopt_long(argc, argv, "a:Ab:B:c:C:dD:e:E:f:Fg:GhH:iIl:Lm:MnNo:O:p:P:q:r:Rs:St:Tu:UvVwW:x:Xz:", l_opts, &option_index)) != EOF){
 #else
-	while ((c=getopt(argc,argv,"a:Ab:B:c:C:de:f:Fg:GhH:iIl:Lm:MnNo:O:p:P:q:r:Rs:t:Tu:UvVwW:x:z")) != EOF){
+	while ((c=getopt(argc, argv, "a:Ab:B:c:C:dD:e:E:f:Fg:GhH:iIl:Lm:MnNo:O:p:P:q:r:Rs:St:Tu:UvVwW:x:z:")) != EOF){
 #endif
 		switch(c){
 			case 'a':
-				password=malloc(strlen(optarg));
+				password=str_alloc(strlen(optarg) + 1);
 				strncpy(password, optarg, strlen(optarg));
-				*(password+strlen(optarg)) = '\0';
 				break;
 			case 'A':
 				timing=1;
 				break;
 			case 'b':
-				if ((namebeg=atoi(optarg))==-1) {
-					printf("error: non-numerical appendix begin for the "
-						"username\n");
-					exit_code(2);
-				}
+				namebeg=str_to_int(optarg);
 				break;
 			case 'B':
-				mes_body=malloc(strlen(optarg));
+				mes_body=str_alloc(strlen(optarg) + 1);
 				strncpy(mes_body, optarg, strlen(optarg));
-				*(mes_body+strlen(optarg)) = '\0';
+				break;
+			case 'c':
+				backup=str_alloc(strlen(optarg)+1);
+				strncpy(backup, optarg, strlen(optarg));
+				parse_uri(backup, &scheme, &user, &host, &port);
+				if (scheme  == NULL) {
+					fprintf(stderr, "error: missing scheme in From URI\n");
+					exit_code(2);
+				}
+				else if (user == NULL) {
+					fprintf(stderr, "error: missing username in From URI\n");
+					exit_code(2);
+				}
+				else if (host == NULL) {
+					fprintf(stderr, "error: missing host in From URI\n");
+					exit_code(2);
+				}
+				else {
+					from_uri=str_alloc(strlen(optarg)+1);
+					strncpy(from_uri, optarg, strlen(optarg));
+				}
+				free(backup);
 				break;
 			case 'C':
-#ifdef HAVE_STRNCASECMP
-				if (!strncasecmp(optarg, "empty", 5) || !strncasecmp(optarg, "none", 4)) {
-#else
-				if (!strncmp(optarg, "empty", 5) || !strncmp(optarg, "none", 4)) {
-#endif
+				if ((strlen(optarg) == 5 && STRNCASECMP(optarg, "empty", 5) == 0) || 
+					(strlen(optarg) == 4 && STRNCASECMP(optarg, "none", 4) == 0)) {
 					empty_contact = 1;
 				}
-				else if (((delim=STRSTR(optarg,"sip:"))!=NULL) ||
-					((delim=STRSTR(optarg,"sips:"))!=NULL)) {
-			 		if (strchr(optarg,'@')<delim) {
-						printf("error: missing '@' in Contact uri\n");
+				else if (strlen(optarg) == 1 && STRNCASECMP(optarg, "*", 1) == 0) {
+					contact_uri=str_alloc(strlen(optarg)+1);
+					strncpy(contact_uri, optarg, strlen(optarg));
+				}
+				else {
+					backup=str_alloc(strlen(optarg)+1);
+					strncpy(backup, optarg, strlen(optarg));
+					parse_uri(backup, &scheme, &user, &host, &port);
+					if (scheme == NULL) {
+					    fprintf(stderr, "error: REGISTER Contact uri doesn't not contain "
+						   "sip:, sips:, *, or is not empty\n");
+				    	exit_code(2);
+					}
+					else if (user == NULL) {
+						fprintf(stderr, "error: missing username in Contact uri\n");
+						exit_code(2);
+					}
+					else if (host == NULL) {
+						fprintf(stderr, "error: missing host in Contact uri\n");
 						exit_code(2);
 					}
 					else {
-						contact_uri=malloc(strlen(optarg)+1);
-						memset(contact_uri, 0, strlen(optarg)+1);
+						contact_uri=str_alloc(strlen(optarg)+1);
 						strncpy(contact_uri, optarg, strlen(optarg));
 					}
-				}
-				else if ((strlen(optarg)==1) && (!strncmp(optarg, "*", 1))) {
-					contact_uri=malloc(strlen(optarg)+1);
-					memset(contact_uri, 0, strlen(optarg)+1);
-					strncpy(contact_uri, optarg, strlen(optarg));
-				}
-				else{
-				    printf("error: REGISTER Contact uri doesn't not contain "
-					   "sip:, *, or is not empty\n");
-				    exit_code(2);
-				}
-				break;
-			case 'c':
-				if ((namebeg=atoi(optarg))==-1) {
-					printf("error: non-numerical CSeq maximum\n");
-					exit_code(2);
+					free(backup);
 				}
 				break;
 			case 'd':
 				redirects=0;
 				break;
+			case 'D':
+				inv_final = str_to_int(optarg) * SIP_T1;
+				break;
 			case 'e':
-				if ((nameend=atoi(optarg))==-1) {
-					printf("error: non-numerical appendix end for the "
-						"username\n");
+				nameend=str_to_int(optarg);
+				break;
+			case 'E':
+				if (strlen(optarg) == 3 && 
+					STRNCASECMP(optarg, "udp", 3) == 0) {
+					transport = SIP_UDP_TRANSPORT;
+				}
+				else if (strlen(optarg) == 3 &&
+						STRNCASECMP(optarg, "tcp", 3) == 0) {
+					transport = SIP_TCP_TRANSPORT;
+				}
+				else if (strlen(optarg) == 3 &&
+						STRNCASECMP(optarg, "tls", 3) == 0) {
+					fprintf(stderr, "error: TLS is not supported yet, supported values: udp, tcp\n");
+					exit_code(2);
+					transport = SIP_TLS_TRANSPORT;
+				}
+				else {
+					fprintf(stderr, "error: unsupported transport '%s', supported values: udp, tcp\n", optarg);
 					exit_code(2);
 				}
 				break;
@@ -363,35 +408,32 @@ int main(int argc, char *argv[])
 				flood=1;
 				break;
 			case 'f':
-				if (strncmp(optarg, "-", 1)) {
+				if (strlen(optarg) != 1 && STRNCASECMP(optarg, "-", 1) != 0) {
 					/* file is opened in binary mode so that the cr-lf is 
 					   preserved */
 					pf = fopen(optarg, "rb");
 					if (!pf){
-						printf("error: unable to open the file '%s'.\n", optarg);
+						fprintf(stderr, "error: unable to open the file '%s'.\n", optarg);
 						exit_code(2);
 					}
-					length  = fread(buff, 1, sizeof(buff), pf);
-					if (length >= sizeof(buff)){
-						printf("error:the file is too big. try files of less "
+					if (fread(buff, 1, sizeof(buff), pf) >= sizeof(buff)){
+						fprintf(stderr, "error:the file is too big. try files of less "
 							"than %i bytes.\n", BUFSIZE);
-						printf("      or recompile the program with bigger "
+						fprintf(stderr, "      or recompile the program with bigger "
 							"BUFSIZE defined.\n");
 						exit_code(2);
 					}
 					fclose(pf);
 				}
-				else {
-					for(i = 0; i < BUFSIZE - 1; i++) {
-						j = getchar();
-						if (j == EOF)
-							break;
-						else
-							buff[i] = j;
+				else if (strlen(optarg) == 1 && STRNCASECMP(optarg, "-", 1) == 0) {
+					if (read_stdin(&buff[0], sizeof(buff)) == 0) {
+						exit_code(0);
 					}
-					length = i;
 				}
-				buff[length] = '\0';
+				else {
+					fprintf(stderr, "error: unable to handle input file name: %s\n", optarg);
+					exit_code(2);
+				}
 				file_b=1;
 				break;
 			case 'g':
@@ -413,21 +455,13 @@ int main(int argc, char *argv[])
 				invite=1;
 				break;
 			case 'l':
-				lport=atoi(optarg);
-				if (!lport) {
-					printf("error: non-numerical local port number");
-					exit_code(2);
-				}
+				lport=str_to_int(optarg);
 				break;
 			case 'L':
 				fix_crlf=0;
 				break;
 			case 'm':
-				maxforw=atoi(optarg);
-				if (maxforw==-1) {
-					printf("error: non-numerical number of max-forwards\n");
-					exit_code(2);
-				}
+				maxforw=str_to_int(optarg);
 				break;
 			case 'M':
 				message=1;
@@ -440,53 +474,49 @@ int main(int argc, char *argv[])
 				break;
 			case 'o':
 				sleep_ms = 0;
-#ifdef HAVE_STRNCASECMP
-				if (strncasecmp(optarg, "rand", 4)==0) {
-#else
-				if (strncmp(optarg, "rand", 4)==0) {
-#endif
+				if (strlen(optarg) == 4 && STRNCASECMP(optarg, "rand", 4) == 0) {
 					sleep_ms = -2;
 				}
 				else {
-					sleep_ms = atoi(optarg);
-					if (!sleep_ms) {
-						printf("error: non-numerical sleep value\n");
-						exit_code(2);
-					}
+					sleep_ms = str_to_int(optarg);
 				}
 				break;
 			case 'O':
-				con_dis=malloc(strlen(optarg));
+				con_dis=str_alloc(strlen(optarg) + 1);
 				strncpy(con_dis, optarg, strlen(optarg));
-				*(con_dis+strlen(optarg)) = '\0';
 				break;
 			case 'p':
-				if ((delim=strchr(optarg,':'))!=NULL){
-					*delim = '\0';
-					delim++;
-					rport = atoi(delim);
+				parse_uri(optarg, &scheme, &user, &host, &rport);
+				if (host == NULL) {
+					fprintf(stderr, "error: missing in host in outbound proxy\n");
+					exit_code(2);
+				}
+				if (is_ip(host)) {
+					address = getaddress(host);
+					if (transport == 0)
+						transport = SIP_UDP_TRANSPORT;
+				}
+				else {
 					if (!rport) {
-						printf("error: non-numerical outbound proxy port "
-							"number\n");
+						address = getsrvadr(host, &rport, &tsp);
+						if (tsp != 0)
+							transport = tsp;
+					}
+					if (!address) {
+						address = getaddress(host);
+						if (address && verbose > 1)
+							printf("using A record: %s\n", host);
+					}
+					if (!address){
+						fprintf(stderr, "error:unable to determine the outbound proxy "
+							"address\n");
 						exit_code(2);
 					}
 				}
-				if (!rport)
-					address = getsrvaddress(optarg, &rport);
-				if (!address)
-					address = getaddress(optarg);
-				if (!address){
-					printf("error:unable to determine the outbound proxy "
-						"address\n");
-					exit_code(2);
-				}
+				outbound_proxy=1;
 				break;
-	        case 'P':
-				processes=atoi(optarg);
-				if (!processes) {
-					printf("error: non-numerical number of processes\n");
-					exit_code(2);
-				}
+			case 'P':
+				processes=str_to_int(optarg);
 				break;
 			case 'q':
 				if (re) {
@@ -506,85 +536,77 @@ int main(int argc, char *argv[])
 				};
 				break;
 			case 'r':
-				rport=atoi(optarg);
-				if (!rport) {
-					printf("error: non-numerical remote port number\n");
-					exit_code(2);
+				port = 0;
+				port=str_to_int(optarg);
+				if (rport) {
+					fprintf(stderr, "warning: you are overwritting the destination port with the r argument\n");
 				}
+				rport = port;
 				break;
 			case 'R':
 				randtrash=1;
 				break;
 			case 's':
-				/* we try to extract as much informationas we can from the uri*/
-#ifdef HAVE_STRNCASECMP
-				if (!strncasecmp(optarg,"sips",4)){
-#else
-				if (!strncmp(optarg,"sips",4)){
-#endif
-					printf("error: sips is not supported yet\n");
+				parse_uri(optarg, &scheme, &user, &host, &port);
+				if (scheme == NULL) {
+					fprintf(stderr, "error: missing scheme in sip uri\n");
 					exit_code(2);
-        }
-#ifdef HAVE_STRNCASECMP
-        else if (!strncasecmp(optarg,"sip",3)){
-#else
-        else if (!strncmp(optarg,"sip",3)){
-#endif
-					if ((delim=strchr(optarg,':'))!=NULL){
-						delim++;
-						if ((delim2=strchr(delim,'@'))!=NULL){
-							username=malloc(delim2-delim+1);
-							strncpy(username, delim, delim2-delim);
-							*(username+(delim2-delim)) = '\0';
-							delim2++;
-							delim=delim2;
-						}
-						if ((delim2=strchr(delim,':'))!=NULL){
-							domainname=malloc(strlen(delim)+1);
-							strncpy(domainname, delim, strlen(delim));
-							*(domainname+strlen(delim)) = '\0';
-							*delim2 = '\0';
-							delim2++;
-							rport = atoi(delim2);
-							if (!rport) {
-								printf("error: non-numerical remote port "
-									"number\n");
-								exit_code(2);
-							}
-						}
-						else {
-							domainname=malloc(strlen(delim)+1);
-							strncpy(domainname, delim, strlen(delim));
-							*(domainname+strlen(delim)) = '\0';
-						}
-						if (!rport && !address)
-							address = getsrvaddress(delim, &rport);
-						if (!address)
-							address = getaddress(delim);
-						if (!address){
-							printf("error:unable to determine the remote host "
-								"address\n");
-							exit_code(2);
-						}
+				}
+				if (strlen(optarg) == 4 && STRNCASECMP(optarg,"sips",4) == 0){
+					fprintf(stderr, "error: sips is not supported yet\n");
+					exit_code(2);
+				}
+				else if (strlen(optarg) != 3 || STRNCASECMP(optarg,"sip",3) != 0){
+					fprintf(stderr, "error: scheme of sip uri has to be sip\n");
+					exit_code(2);
+				}
+				if (user != NULL) {
+					username = user;
+				}
+				if (host != NULL) {
+					domainname = host;
+				}
+				else {
+					fprintf(stderr, "error: missing hostname in sip uri\n");
+					exit_code(2);
+				}
+				if (port && !rport) {
+					rport = port;
+				}
+				if (is_ip(domainname)) {
+					address = getaddress(domainname);
+					if (transport == 0)
+						transport = SIP_UDP_TRANSPORT;
+				}
+				else {
+					if (!rport && !address) {
+						address = getsrvadr(domainname, &rport, &tsp);
+						if (tsp != 0)
+							transport = tsp;
 					}
-					else{
-						printf("error: SIPURI doesn't contain a : ?!\n");
+					if (!address) {
+						address = getaddress(domainname);
+						if (address && verbose > 1)
+							printf("using A record: %s\n", domainname);
+					}
+					if (!address){
+						fprintf(stderr, "error:unable to determine the IP address for: %s\n", domainname);
 						exit_code(2);
 					}
 				}
-				else{
-					printf("error: SIPURI doesn't not begin with sip\n");
-					exit_code(2);
+				if (port != 0) {
+					backup = str_alloc(strlen(domainname)+1+6);
+					snprintf(backup, strlen(domainname)+6, "%s:%i", domainname, port);
+					domainname = backup;
 				}
 				uri_b=1;
-				break;			break;
+				break;
+			case 'S':
+				fprintf(stderr, "warning: symmetric does not work with a-symmetric servers\n");
+				symmetric=1;
+				break;
 			case 't':
-				trashchar=atoi(optarg);
-				if (!trashchar) {
-					printf("error: non-numerical number of trashed "
-						"character\n");
-					exit_code(2);
-				}
+				trashchar=str_to_int(optarg);
 				break;
 			case 'T':
 				trace=1;
@@ -593,9 +615,8 @@ int main(int argc, char *argv[])
 				usrloc=1;
 				break;
 			case 'u':
-				auth_username=malloc(strlen(optarg));
+				auth_username=str_alloc(strlen(optarg) + 1);
 				strncpy(auth_username, optarg, strlen(optarg));
-				*(auth_username+strlen(optarg)) = '\0';
 				break;
 			case 'v':
 				verbose++;
@@ -604,25 +625,35 @@ int main(int argc, char *argv[])
 				printf("sipsak %s  by Nils Ohlmeier\n Copyright (C) 2002-2004"
 						" FhG Fokus\n Copyright (C) 2004-2005 Nils Ohlmeier\n", 
 						SIPSAK_VERSION);
-				printf(" compiled with DEFAULT_RETRYS=%i, DEFAULT_TIMEOUT=%i, FQDN_SIZE=%i",
-						DEFAULT_RETRYS, DEFAULT_TIMEOUT, FQDN_SIZE);
+				printf(" compiled with DEFAULT_TIMEOUT=%i, FQDN_SIZE=%i",
+						DEFAULT_TIMEOUT, FQDN_SIZE);
 #ifdef RAW_SUPPORT
 				printf(", RAW_SUPPORT");
 #endif
 #ifdef HAVE_GETOPT_LONG
 				printf(", LONG_OPTS");
 #endif
-#ifdef HAVE_OPENSSL_MD5_H
+#ifdef HAVE_GNUTLS
+				printf(", GNUTLS_MD5");
+#else
+# ifdef HAVE_FULL_OPENSSL
 				printf(", OPENSSL_MD5");
+# else
+				printf(", INTERNAL_MD5");
+# endif
 #endif
-#ifdef HAVE_RULI_H
-				printf(", SRV_SUPPORT");
+#ifdef HAVE_CARES_H
+				printf(", SRV_SUPPORT(ARES)");
+#else
+# ifdef HAVE_RULI_H
+				printf(", SRV_SUPPORT(RULI)");
+# endif
 #endif
 #ifdef HAVE_STRCASESTR
-        printf(", STR_CASE_INSENSITIVE");
+				printf(", STR_CASE_INSENSITIVE");
 #endif
 #ifdef HAVE_STRNCASECMP
-        printf(", CMP_CASE_INSENSITIVE");
+				printf(", CMP_CASE_INSENSITIVE");
 #endif
 				printf("\n");
 				exit_code(0);
@@ -631,10 +662,10 @@ int main(int argc, char *argv[])
 				warning_ext=1;
 				break;
 			case 'W':
-				nagios_warn = atoi(optarg);
+				nagios_warn = str_to_int(optarg);
 				break;
 			case 'x':
-				expires_t=atoi(optarg);
+				expires_t=str_to_int(optarg);
 				break;
 #ifdef HAVE_GETOPT_LONG
 			case 'X':
@@ -642,10 +673,14 @@ int main(int argc, char *argv[])
 				break;
 #endif
 			case 'z':
-				rand_rem=1;
+				rand_rem=str_to_int(optarg);
+				if (rand_rem < 0 || rand_rem > 100) {
+					fprintf(stderr, "error: z option must between 0 and 100\n");
+					exit_code(2);
+				}
 				break;
 			default:
-				printf("error: unknown parameter %c\n", c);
+				fprintf(stderr, "error: unknown parameter %c\n", c);
 				exit_code(2);
 				break;
 		}
@@ -654,6 +689,13 @@ int main(int argc, char *argv[])
 	if (rport == 0) {
 		rport =  5060;
 	}
+	if (rport > 65535 || rport <= 0) {
+		fprintf(stderr, "error: invalid remote port: %i\n", rport);
+		exit_code(2);
+	}
+	if (transport == 0) {
+		transport = SIP_UDP_TRANSPORT;
+	}
 
 	/* replace LF with CRLF if we read from a file */
 	if ((file_b) && (fix_crlf)) {
@@ -662,28 +704,28 @@ int main(int argc, char *argv[])
 	/* lots of conditions to check */
 	if (trace) {
 		if (usrloc || flood || randtrash) {
-			printf("error: trace can't be combined with usrloc, random or "
+			fprintf(stderr, "error: trace can't be combined with usrloc, random or "
 				"flood\n");
 			exit_code(2);
 		}
 		if (!uri_b) {
-			printf("error: for trace mode a SIPURI is realy needed\n");
+			fprintf(stderr, "error: for trace mode a SIPURI is realy needed\n");
 			exit_code(2);
 		}
 		if (file_b) {
-			printf("warning: file will be ignored for tracing.");
+			fprintf(stderr, "warning: file will be ignored for tracing.");
 		}
 		if (!username) {
-			printf("error: for trace mode without a file the SIPURI have to "
+			fprintf(stderr, "error: for trace mode without a file the SIPURI have to "
 				"contain a username\n");
 			exit_code(2);
 		}
 		if (!via_ins){
-			printf("warning: Via-Line is needed for tracing. Ignoring -i\n");
+			fprintf(stderr, "warning: Via-Line is needed for tracing. Ignoring -i\n");
 			via_ins=1;
 		}
 		if (!warning_ext) {
-			printf("warning: IP extract from warning activated to be more "
+			fprintf(stderr, "warning: IP extract from warning activated to be more "
 				"informational\n");
 			warning_ext=1;
 		}
@@ -691,48 +733,48 @@ int main(int argc, char *argv[])
 	}
 	else if (usrloc || invite || message) {
 		if (trace || flood || randtrash) {
-			printf("error: usrloc can't be combined with trace, random or "
+			fprintf(stderr, "error: usrloc can't be combined with trace, random or "
 				"flood\n");
 			exit_code(2);
 		}
 		if (!username || !uri_b) {
-			printf("error: for the USRLOC mode you have to give a SIPURI with "
+			fprintf(stderr, "error: for the USRLOC mode you have to give a SIPURI with "
 				"a username\n       at least\n");
 			exit_code(2);
 		}
 		if (namebeg>0 && nameend==-1) {
-			printf("error: if a starting numbers is given also an ending "
+			fprintf(stderr, "error: if a starting numbers is given also an ending "
 				"number have to be specified\n");
 			exit_code(2);
 		}
 		if (invite && message) {
-			printf("error: invite and message tests are XOR\n");
+			fprintf(stderr, "error: invite and message tests are XOR\n");
 			exit_code(2);
 		}
 		if (!usrloc && invite && !lport) {
-			printf("WARNING: Do NOT use the usrloc invite mode without "
+			fprintf(stderr, "warning: Do NOT use the usrloc invite mode without "
 				"registering sipsak before.\n         See man page for "
 				"details.\n");
 			exit_code(2);
 		}
 		if (contact_uri!=NULL) {
 			if (invite || message) {
-				printf("error: Contact uri is not support for invites or "
+				fprintf(stderr, "error: Contact uri is not support for invites or "
 					"messages\n");
 				exit_code(2);
 			}
 			if (nameend!=-1 || namebeg!=-1) {
-				printf("warning: ignoring starting or ending number if Contact"
+				fprintf(stderr, "warning: ignoring starting or ending number if Contact"
 					" is given\n");
 				nameend=namebeg=0;
 			}
 			if (rand_rem) {
-				printf("warning: ignoring -z option when Contact is given\n");
+				fprintf(stderr, "warning: ignoring -z option when Contact is given\n");
 				rand_rem=0;
 			}
 		}
 		if (via_ins) {
-			printf("warning: ignoring -i option when in usrloc mode\n");
+			fprintf(stderr, "warning: ignoring -i option when in usrloc mode\n");
 			via_ins=0;
 		}
 		if (nameend==-1)
@@ -742,47 +784,47 @@ int main(int argc, char *argv[])
 	}
 	else if (flood) {
 		if (trace || usrloc || randtrash) {
-			printf("error: flood can't be combined with trace, random or "
+			fprintf(stderr, "error: flood can't be combined with trace, random or "
 				"usrloc\n");
 			exit_code(2);
 		}
 		if (!uri_b) {
-			printf("error: we need at least a sip uri for flood\n");
+			fprintf(stderr, "error: we need at least a sip uri for flood\n");
 			exit_code(2);
 		}
 		if (redirects) {
-			printf("warning: redirects are not expected in flood. "
+			fprintf(stderr, "warning: redirects are not expected in flood. "
 				"disableing\n");
 			redirects=0;
 		}
 	}
 	else if (randtrash) {
 		if (trace || usrloc || flood) {
-			printf("error: random can't be combined with trace, flood or "
+			fprintf(stderr, "error: random can't be combined with trace, flood or "
 				"usrloc\n");
 			exit_code(2);
 		}
 		if (!uri_b) {
-			printf("error: need at least a sip uri for random\n");
+			fprintf(stderr, "error: need at least a sip uri for random\n");
 			exit_code(2);
 		}
 		if (redirects) {
-			printf("warning: redirects are not expected in random. "
+			fprintf(stderr, "warning: redirects are not expected in random. "
 				"disableing\n");
 			redirects=0;
 		}
 		if (verbose) {
-			printf("warning: random characters may destroy your terminal "
+			fprintf(stderr, "warning: random characters may destroy your terminal "
 				"output\n");
 		}
 	}
 	else if (mes_body) {
 		if (!message) {
-			printf("warning: to send a message mode (-M) is required. activating\n");
+			fprintf(stderr, "warning: to send a message mode (-M) is required. activating\n");
 			message=1;
 		}
 		if (!uri_b) {
-			printf("error: need at least a sip uri to send a meesage\n");
+			fprintf(stderr, "error: need at least a sip uri to send a meesage\n");
 			exit_code(2);
 		}
 		if (nameend==-1)
@@ -792,10 +834,26 @@ int main(int argc, char *argv[])
 	}
 	else {
 		if (!uri_b) {
-			printf("error: a spi uri is needed at least\n");
+			fprintf(stderr, "error: a spi uri is needed at least\n");
 			exit_code(2);
 		}
 	}
+
+	switch (transport) {
+		case SIP_TLS_TRANSPORT:
+			transport_str = TRANSPORT_TLS_STR;
+			break;
+		case SIP_TCP_TRANSPORT:
+			transport_str = TRANSPORT_TCP_STR;
+			break;
+		case SIP_UDP_TRANSPORT:
+			transport_str = TRANSPORT_UDP_STR;
+			break;
+		default:
+			fprintf(stderr, "unknown transport: %i\n", transport);
+			exit_code(2);
+	}
+
 	/* determine our hostname */
 	get_fqdn();
 	
@@ -805,14 +863,14 @@ int main(int argc, char *argv[])
 	
 	if (processes > 1) {
 		if (signal(SIGCHLD , sigchld_handler)  == SIG_ERR ) {
-			printf("error: Could not install SIGCHLD handler\n");
+			fprintf(stderr, "error: Could not install SIGCHLD handler\n");
 			exit_code(2);
 		}
 	}
 
 	for(i = 0; i < processes - 1; i++) {
 		if ((pid = fork()) < 0) {
-			printf("error: Cannot fork\n");
+			fprintf(stderr, "error: Cannot fork\n");
 			exit_code(2);
 		}
 		
@@ -821,7 +879,7 @@ int main(int argc, char *argv[])
 			upp = (nameend - namebeg + 1) / processes;
 			namebeg = namebeg + upp * i;
 			nameend = namebeg + upp;
-			shoot(buff);
+			shoot(&buff[0], sizeof(buff));
 		} else {
 			if (lport) {
 				lport++;
@@ -843,7 +901,7 @@ int main(int argc, char *argv[])
 		namebeg = namebeg + upp * i;
 		nameend = namebeg + upp;
 	}
-	shoot(buff);
+	shoot(&buff[0], sizeof(buff));
 
 	/* normaly we won't come back here, but to satisfy the compiler */
 	return 0;
